package com.ibm.websphere.samples.cmpa;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.7 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/AccountBackendBean.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 4/13/06 22:09:58 [8/24/10 06:41:33]
//
//==============================================================================

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.security.AccessControlException;
import java.util.logging.Logger;
import java.util.Vector;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;


/**
 * Bean implementation class for Enterprise Bean: AccountBackend
 */
public class AccountBackendBean implements SessionBean {
	private SessionContext mySessionCtx;
	private static File dataDir = null;
	private Logger log = Logger.getLogger("com.ibm.websphere.samples.cmpa");

	/**
	 * getSessionContext
	 */
	public SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(SessionContext ctx) {
		mySessionCtx = ctx;

		if (dataDir == null) {
			log.entering("AccountBackendBean", "setSessionContext(SessionContext)", ctx);

         //This EJB persists data to files in the profile's samples/CMPa dir
         //The name of each file is the primary key (the account ID string),
         //and the contents of the file is the other CMP field (the balance)
			dataDir = new File("samples"+File.separator+"CMPa");

			try {
				if (!dataDir.exists()) { //need was.policy file to avoid exceptions
					dataDir.mkdirs();     //if Java2 security is enabled
				}
			} catch (AccessControlException ace) {
				log.throwing("AccountBackendBean", "setSessionContext(SessionContext)", ace);

				throw ace;
			}

			log.exiting("AccountBackendBean", "setSessionContext(SessionContext)");
		}
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

	public void createAccount(String id, Float balance) throws DuplicateKeyException {
		log.entering("AccountBackendBean", "createAccount(String, Float)", new Object[] {id, balance});

		File account = new File(dataDir, id);
		if (account.exists()) {
			DuplicateKeyException dupKey = new DuplicateKeyException(id);

			log.throwing("AccountBackendBean", "createAccount(String, Float)", dupKey);

			throw dupKey;
		} else {
			try {
				writeFileContents(account, balance);
			} catch (IOException ioe) {
				EJBException ejbEx = new EJBException(ioe);

				log.throwing("AccountBackendBean", "createAccount(String, Float)", ejbEx);

				throw ejbEx;
			}
		}

		log.exiting("AccountBackendBean", "createAccount(String, Float)");
	}

	//Demonstrate returning a custom type	
	public AccountWrapper retrieveAccount(String id) throws ObjectNotFoundException {
		log.entering("AccountBackendBean", "retrieveAccount(String)", id);

		Float balance = null;
		AccountWrapper wrapper = null;

		File account = new File(dataDir, id);
		if (account.exists()) {
			try {
				String contents = readFileContents(account);
				balance = new Float(contents);
			} catch (Exception e) {
				EJBException ejbEx = new EJBException(e);

				log.throwing("AccountBackendBean", "retrieveAccount(String)", ejbEx);

				throw ejbEx;
			}
			wrapper = new AccountWrapper(id, balance);
		} else {
			ObjectNotFoundException notFound = new ObjectNotFoundException(id);

			log.throwing("AccountBackendBean", "retrieveAccount(String)", notFound);

			throw notFound;
		}

		log.exiting("AccountBackendBean", "retrieveAccount(String)", wrapper);

		//Note that the JAX-RPC version of the sample will fail here, when
		//de-marshalling this complex type, if "Deploy Web Services" was not
		//selected during installation of the cmpaSample.ear (this option is
		//automatically selected if the sample is installed via the WBI Samples
		//Gallery's "installwbi -samples CMPa" script.
		return wrapper;
	}

	public boolean updateAccount(String id, Float balance) {
		log.entering("AccountBackendBean", "updateAccount(String, Float)", new Object[] {id, balance});

		boolean error = false;

		File account = new File(dataDir, id);
		if (account.exists()) {
			try {
				writeFileContents(account, balance);
			} catch (IOException ioe) {
				EJBException ejbEx = new EJBException(ioe);

				log.throwing("AccountBackendBean", "updateAccount(String, Float)", ejbEx);

				throw ejbEx;
			}
		} else {
			error = true;
		}

		log.exiting("AccountBackendBean", "updateAccount(String, Float)", new Boolean(error));

		return error;
	}

	public boolean deleteAccount(String id) {
		log.entering("AccountBackendBean", "deleteAccount(String)", id);

		boolean error = false;

		File account = new File(dataDir, id);
		if (account.exists()) {
			error = !account.delete();
		} else {
			error = true;
		}

		log.exiting("AccountBackendBean", "deleteAccount(String)", new Boolean(error));

		return error;
	}

	public Vector selectAccounts() {
		log.entering("AccountBackendBean", "selectAccounts()");

		//Made this a Vector to avoid validation warning about the Collection
		//interface (implemented by Vector) not being serializable.
		Vector balances = new Vector();

		Float balance = null;
		File accounts[] = dataDir.listFiles();
		for (int index=0; index<accounts.length; index++) {
			File account = accounts[index];
			try {
				String contents = readFileContents(account);
				balance = new Float(contents);
				balances.add(balance);
			} catch (Exception e) {
				EJBException ejbEx = new EJBException(e);

				log.throwing("AccountBackendBean", "selectAccounts()", ejbEx);

				throw ejbEx;
			}
		}

		log.exiting("AccountBackendBean", "selectAccounts()", balances);

		return balances;
	}

	public Vector retrieveMultipleAccounts(float threshold) {
		log.entering("AccountBackendBean", "retrieveMultipleAccount(float)", new Float(threshold));

		//Made this a Vector to avoid validation warning about the Collection
		//interface (implemented by Vector) not being serializable.
		Vector accounts = new Vector();

		Float balance = null;
		File allAccounts[] = dataDir.listFiles();
		for (int index=0; index<allAccounts.length; index++) {
			File account = allAccounts[index];
			try {
				String contents = readFileContents(account);
				balance = new Float(contents);
				if (balance.floatValue() >= threshold) {
					String id = account.getName();
					AccountWrapper wrapper = new AccountWrapper(id, balance);
					accounts.add(wrapper);
				}
			} catch (Exception e) {
				EJBException ejbEx = new EJBException(e);

				log.throwing("AccountBackendBean", "retrieveMultipleAccount(float)", ejbEx);

				throw ejbEx;
			}
		}

		log.exiting("AccountBackendBean", "retrieveMultipleAccount(float)", accounts);

		return accounts;
	}

	public float creditAccount(String id, float amount)
		throws ObjectNotFoundException {

		log.entering("AccountBackendBean", "creditAccount(String, float)", new Object[] {id, new Float(amount)});

		float newBalance = 0;
		File account = new File(dataDir, id);
		if (account.exists()) {
			try {
				String contents = readFileContents(account);
				float oldBalance = Float.parseFloat(contents);
				newBalance = oldBalance + amount;
				writeFileContents(account, new Float(newBalance));
			} catch (Exception e) {
				EJBException ejbEx = new EJBException(e);

				log.throwing("AccountBackendBean", "creditAccount(String, float)", ejbEx);

				throw ejbEx;
			}
		} else {
			ObjectNotFoundException notFound = new ObjectNotFoundException(id);

			log.throwing("AccountBackendBean", "creditAccount(String, float)", notFound);

			throw notFound;
		}

		log.exiting("AccountBackendBean", "creditAccount(String, float)", new Float(newBalance));

		return newBalance;
	}

	public float debitAccount(String id, float amount)
		throws ObjectNotFoundException, InsufficientFundsException {

		log.entering("AccountBackendBean", "debitAccount(String, float)", new Object[] {id, new Float(amount)});

		float newBalance = 0;
		File account = new File(dataDir, id);
		if (account.exists()) {
			InsufficientFundsException overdrawn = null;

			try {
				String contents = readFileContents(account);
				float oldBalance = Float.parseFloat(contents);
				if (oldBalance >= amount) {
					newBalance = oldBalance - amount;
					writeFileContents(account, new Float(newBalance));
				} else {
					overdrawn = new InsufficientFundsException(""+oldBalance);
				}
			} catch (Exception e) {
				EJBException ejbEx = new EJBException(e);

				log.throwing("AccountBackendBean", "debitAccount(String, float)", ejbEx);

				throw ejbEx;
			}

			if (overdrawn != null) {
				log.throwing("AccountBackendBean", "debitAccount(String, float)", overdrawn);

				throw overdrawn;
			}
		} else {
			ObjectNotFoundException notFound = new ObjectNotFoundException(id);

			log.throwing("AccountBackendBean", "debitAccount(String, float)", notFound);

			throw notFound;
		}

		log.exiting("AccountBackendBean", "debitAccount(String, float)", new Float(newBalance));

		return newBalance;
	}

	public void resetAccounts() {
		log.entering("AccountBackendBean", "resetAccounts()");

		File accounts[] = dataDir.listFiles();
		for (int index=0; index<accounts.length; index++) {
			File account = accounts[index];
			account.delete();
		}

		log.exiting("AccountBackendBean", "resetAccounts()");
	}

	//read the contents of a file and return in a big String
	public static String readFileContents(File file) throws IOException {
		int fileLength = (int) file.length();
		char fileChars[] = new char[fileLength];

		FileReader fileReader = new FileReader(file);
		fileReader.read(fileChars, 0, fileLength);
		fileReader.close(); //d269894

		String fileContents = new String(fileChars);
		return fileContents;
	}

	//write the contents of a file and based on the input Object
	public static void writeFileContents(File file, Object contents) throws IOException {
		FileWriter fileWriter = new FileWriter(file);
		fileWriter.write(contents.toString());
		fileWriter.close();
	}
}
