package com.ibm.websphere.samples.cmpa;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-H74,   5724-L01, 5655-N53, 5724-I82, 5655-R15 (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.7 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/AccountClient.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 4/18/07 12:00:20 [8/24/10 06:41:33]
//
//==============================================================================

import java.util.Collection;
import java.util.Iterator;
import java.util.ResourceBundle;
import javax.ejb.DuplicateKeyException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;


/**
 * Command-line J2EE application client for the CMP/A sample
 */
public class AccountClient {
   private static ResourceBundle bundle = null;

	private static int FAILURE = -1;
	private static int SUCCESS = 0;
	private static int USER_EXCEPTION = 1;
	private static int DUP_KEY = 2;
	private static int NOT_FOUND = 3;
	private static int REMOVE_FAILED = 4;
	

	public static void main(String[] args) {
		boolean find = false;
		boolean create = false;
		boolean update = false;
		boolean remove = false;
		boolean total = false;
		boolean multiFind = false;
		boolean debit = false;
		boolean credit = false;
		boolean reset = false;
		String key = null;

		if (args.length==3) {
			if (args[1].equalsIgnoreCase("find")) find = true;
			if (args[1].equalsIgnoreCase("create")) create = true;
			if (args[1].equalsIgnoreCase("update")) update = true;
			if (args[1].equalsIgnoreCase("remove")) remove = true;
			if (args[1].equalsIgnoreCase("multiFind")) multiFind = true;
			if (args[1].equalsIgnoreCase("credit")) credit = true;
			if (args[1].equalsIgnoreCase("debit")) debit = true;
			key = args[2];
		} else {
			if ((args.length==2) && args[1].equalsIgnoreCase("reset")) {
				reset = true;
			} else
			if ((args.length==2) && args[1].equalsIgnoreCase("total")) {
				total = true;
			} else {
				print("WRN_USAGE");
				System.exit(FAILURE);
			}
		}
		UserTransaction tran = null;
		try {
			print("INF_SAMPLE");
			InitialContext ctx = new InitialContext();
			
			print("INF_GET_TRAN");
			tran = (UserTransaction) ctx.lookup("java:comp/UserTransaction");
			
			print("INF_GET_HOME");
			Object obj = ctx.lookup("java:comp/env/ejb/Account"+args[0]);
			AccountHome home = (AccountHome) PortableRemoteObject.narrow(obj, AccountHome.class);

			print("INF_BEGIN_TRAN");
			tran.begin();

			Account bean = null;
			float balance = 0;

			if (create) {			
				print("INF_CREATE_ACCOUNT", (float)Math.PI);
				bean = home.create(key, (float)Math.PI);
			} else

			if (find) {
				print("INF_RETRIEVE_ACCOUNT");
				bean = home.findByPrimaryKey(key);

				print("INF_GET_BALANCE");
				balance = bean.getBalance();
				print("INF_ACCOUNT_BALANCE", balance);
			} else

			if (multiFind) {
				print("INF_CUSTOM_FINDER");
				float threshold = Float.parseFloat(key);
				Collection beans = home.findByBalanceThreshold(threshold);
				Iterator iter = beans.iterator();
				while (iter.hasNext()) {
					bean = (Account) iter.next();
					String id = bean.getId();
					balance = bean.getBalance();
					print("INF_KEY_AND_BALANCE", id, balance);
				}

			} else
			
			if (update) {
				print("INF_RETRIEVE_ACCOUNT");
				bean = home.findByPrimaryKey(key);

				print("INF_UPDATE_ACCOUNT", 42f);
				bean.setBalance(42f);
			} else

			if (remove) {
				print("INF_RETRIEVE_ACCOUNT");
				bean = home.findByPrimaryKey(key);

				print("INF_DELETE_ACCOUNT");
				if (key.equals("fail")) {
					print("WRN_REMOVE_EXCEPTION");
				}
				bean.remove();
			} else

			if (credit) {
				print("INF_RETRIEVE_ACCOUNT");
				bean = home.findByPrimaryKey(key);

				print("INF_CREDIT_ACCOUNT", 5f);
				balance = bean.credit(5f);
				print("INF_ACCOUNT_BALANCE", balance);
			} else

			if (debit) {
				print("INF_RETRIEVE_ACCOUNT");
				bean = home.findByPrimaryKey(key);

				print("INF_DEBIT_ACCOUNT", 5f);
				print("WRN_INSUFFICIENT_FUNDS", 5f);
				balance = bean.debit(5f);
				print("INF_ACCOUNT_BALANCE", balance);
			} else

			if (total) {			
				print("INF_SELECT_ACCOUNTS");
				float totalBalance = home.computeTotalBalances();
				print("INF_TOTAL_BALANCE", totalBalance);
			} else

			if (reset) {			
				print("INF_RESET_ACCOUNTS");
				home.reset();
				print("INF_ALL_REMOVED");
			} else {
				print("WRN_USAGE");
				System.exit(FAILURE);
			}

			print("INF_COMMIT_TRAN");
			tran.commit();
			tran = null;

			print("INF_SUCCESS");
			System.exit(SUCCESS);
		} catch (InsufficientFundsException overdrawn) {
			print("ERR_CAUGHT_EXCEPTION", overdrawn);
			System.exit(USER_EXCEPTION);
		} catch (DuplicateKeyException dupKey) {
			print("ERR_CAUGHT_EXCEPTION", dupKey);
			System.exit(DUP_KEY);
		} catch (ObjectNotFoundException notFound) {
			print("ERR_CAUGHT_EXCEPTION", notFound);
			System.exit(NOT_FOUND);
		} catch (RemoveException removeFail) {
			print("ERR_CAUGHT_EXCEPTION", removeFail);
			System.exit(REMOVE_FAILED);
		} catch (Throwable t) {
			t.printStackTrace();
			System.exit(FAILURE);
		} finally {
			if (tran != null) try {
				print("WRN_ROLLBACK_TRAN");
				tran.rollback();
			} catch (Throwable t2) {}
		}
	}

	public static String getResourceString(String key) {
		if (bundle == null) {
			bundle = ResourceBundle
					.getBundle("com.ibm.websphere.samples.cmpa.sampcmpaPIIMessages");
		}

		return bundle.getString(key);
	}

	public static String getResourceString(String key, String substitutionValue) {
		String msg = getResourceString(key);

		int braceLocation = msg.indexOf("{0}");
		if (braceLocation != -1) msg = msg.substring(0, braceLocation)
			+ substitutionValue + msg.substring(braceLocation+3);

		return msg;
	}

	public static String getResourceString(String key, String substitutionValue1, String substitutionValue2) {
		String msg = getResourceString(key, substitutionValue1);

		int braceLocation = msg.indexOf("{1}");
		if (braceLocation != -1) msg = msg.substring(0, braceLocation)
			+ substitutionValue2 + msg.substring(braceLocation+3);

		return msg;
	}

	public static void print(String key) {
		System.out.println(getResourceString(key));
	}

	public static void print(String key, String substitutionValue) {
		System.out.println(getResourceString(key, substitutionValue));
	}

	public static void print(String key, String substitutionValue1, String substitutionValue2) {
		System.out.println(getResourceString(key, substitutionValue1, substitutionValue2));
	}

	public static void print(String key, float amount) {
		String amountString = Float.toString(amount);
		print(key, amountString);
	}

	public static void print(String key, String substitutionValue1, float amount) {
		String amountString = Float.toString(amount);
		print(key, substitutionValue1, amountString);
	}

	public static void print(String key, Throwable t) {
		String throwableString = t.toString();

		int index = throwableString.lastIndexOf(".");
		if (Character.isDigit(throwableString.charAt(index-1))) {
			//handle exception message containing a decimal point
			index = throwableString.lastIndexOf(".", index-1);
		}

		throwableString = throwableString.substring(index+1);
		print(key, throwableString);
	}
}
