package com.ibm.websphere.samples.cmpa.sdorep.rar;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.7 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/sdorep/rar/SdoRepositoryInteractionImpl.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 11/15/07 14:44:42 [8/24/10 06:41:34]
//
//==============================================================================

import com.ibm.wbiserver.cmpa.WSPushDownHelper;
import com.ibm.wbiserver.cmpa.WSPushDownHelperFactory;
import com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.ejb.EJBException;
import javax.ejb.NoSuchEntityException;

import javax.resource.cci.Connection;
import javax.resource.cci.IndexedRecord;
import javax.resource.cci.Interaction;
import javax.resource.cci.InteractionSpec;
import javax.resource.cci.Record;
import javax.resource.cci.ResourceWarning;

import javax.resource.NotSupportedException;
import javax.resource.ResourceException;


/**
 * This class provides an implementation of the CCI Interaction interface, for
 * use by the SDO Repository adapter.  This interaction implementation
 * uses the com.ibm.wsspi.sdo.config.repository.SDORepository SPI (indirectly,
 * via the SdoRepositoryWrapper) to communicate with the actual repository.
 *
 * Note that the SDO Repository is currently implemented as a session bean that
 * talks to a CMP entity bean, though such details are hidden from users of the
 * SPI (and are subject to change); the important thing here is that the SDO
 * Repository is a transaction datastore (supporting two-phase commit if
 * configured with an XA datasource), meaning that a transaction rollback will
 * be honored for CMP/A beans using this adapter.  The SdoRepository.ear must
 * be installed for this adapter to work.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 */
public class SdoRepositoryInteractionImpl implements Interaction {

   private static final Class currClass        = SdoRepositoryInteractionImpl.class;

   private static SdoRepositoryWrapper wrapper = null;
   private static WSPushDownHelper helper      = null;

   private Connection ivConnection             = null;
   private Logger log                          = null;


   /**
    * Constructor
    *
    * @param connection CCI Connection
    */
   public SdoRepositoryInteractionImpl(Connection connection) {
      log = Logger.getLogger("com.ibm.websphere.samples.cmpa.sdorep.rar");
      log.entering("SdoRepositoryInteractionImpl",
         "SdoRepositoryInteractionImpl(Connection)", connection);

      setConnection(connection);

      if (helper == null) {
         helper = WSPushDownHelperFactory.createWSPushDownHelper("SdoRepositoryInteractionImpl");

         //We'll get exceptions if the SdoRepository.ear isn't installed
         try {
            wrapper = new SdoRepositoryWrapper();
         } catch (Throwable t) {
            log.throwing("SdoRepositoryInteractionImpl", "SdoRepositoryInteractionImpl", t);
            t.printStackTrace();
         }
      }

      log.exiting("SdoRepositoryInteractionImpl",
         "SdoRepositoryInteractionImpl(Connection)", null);
   }


   /**
    * Return the CCI Connection object associated with this Interaction.
    *
    * @return javax.resource.cci.Connection see J2EE Connector spec, CCI
    */
   public Connection getConnection() {
      log.entering("SdoRepositoryInteractionImpl", "getConnection()");

      log.exiting("SdoRepositoryInteractionImpl", "getConnection()", ivConnection);

      return ivConnection;
   }


   /**
    * Set the CCI Connection object associated with this Interaction.
    *
    * @param connection see J2EE Connector spec, CCI
    */
   public void setConnection(Connection connection) {
      log.entering("SdoRepositoryInteractionImpl", "setConnection(Connection)", connection);

      ivConnection = connection;

      log.exiting("SdoRepositoryInteractionImpl", "setConnection(Connection)");
   }


   /**
    * Close the interaction.
    *
    * @exception javax.resource.ResourceException
    */
   public void close() throws ResourceException {
      log.entering("SdoRepositoryInteractionImpl", "close()");

      //nothing to do

      log.exiting("SdoRepositoryInteractionImpl", "close()");
   }


   /**
    * Executes an interaction represented by the InteractionSpec. This form of invocation
    * takes an input Record and returns an output Record if the execution of the Interaction
    * has been successful.
    *
    * @param iSpec InteractionSpec representing a target EIS data/function module
    * @param input Input CCI Record
    *
    * @return output Record if execution of the EIS function has been successful; null
    *         otherwise
    *
    * @throws ResourceException Exception if execute operation fails.
    */
   public Record execute(InteractionSpec iSpec, Record input) throws ResourceException {
      log.entering("SdoRepositoryInteractionImpl",
         "execute(InteractionSpec, Record)", new Object[] {iSpec, input});

      Record output = null;

      try {
         IndexedRecord iRecord = (IndexedRecord) input;
         SdoRepositoryInteractionSpec sdoISpec = (SdoRepositoryInteractionSpec) iSpec;
         String functionName = sdoISpec.getFunctionName();

         // Generically useful operations
         if (functionName.equals(SdoRepositoryInteractionSpec.CREATE)) {
            String key = (String) iRecord.get(0);
            Object obj = iRecord.get(1);

            wrapper.ejbCreate(key, obj);
         } else if (functionName.equals(SdoRepositoryInteractionSpec.RETRIEVE)) {
            String key = (String) iRecord.get(0);
            Class type = null;
            if (iRecord.size() == 2) {
               //class type is an optional parameter - only required if the
               //class being found isn't available to the RAR's class loader
               type = (Class) iRecord.get(1);
            }

            Object obj = wrapper.ejbFindByPrimaryKey(key, type);

            output = helper.createCCIRecord(obj);
         } else if (functionName.equals(SdoRepositoryInteractionSpec.UPDATE)) {
            String key = (String) iRecord.get(0);
            Object obj = iRecord.get(1);

            wrapper.ejbStore(key, obj);
         } else if (functionName.equals(SdoRepositoryInteractionSpec.DELETE)) {
            String key = (String) iRecord.get(0);

            wrapper.ejbRemove(key);
         } else if (functionName.equals(SdoRepositoryInteractionSpec.ALL_KEYS)) {
            List keys = wrapper.ejbFindAll();

            output = helper.createCCIIndexedRecord();

            Iterator iter = keys.iterator();
            while (iter.hasNext()) {
               String key = (String) iter.next();
               ((IndexedRecord) output).add(key);
            }
         } else if (functionName.equals(SdoRepositoryInteractionSpec.RESET)) {
            List keys = wrapper.ejbFindAll();

            Iterator iter = keys.iterator();
            while (iter.hasNext()) {
               String key = (String) iter.next();

               wrapper.ejbRemove(key);
            }
         } else {
            throw new NotSupportedException(functionName);
         }
      } catch (ResourceException re) {
         log.throwing("SdoRepositoryInteractionImpl", "execute(InteractionSpec, Record)", re);
         throw re;
      } catch (NoSuchEntityException notThere) {
         //Have to explicitly handle this because it's a subclass of EJBException

         //Any exceptions thrown from the wrapper methods are caught here and
         //stuffed inside a ResourceException that is thrown back to the caller
         log.throwing("SdoRepositoryInteractionImpl", "execute(InteractionSpec, Record)", notThere);
         throw helper.createResourceException(notThere, currClass);
      } catch (EJBException ejb) {
         //Any exceptions thrown from the wrapper methods are caught here and
         //stuffed inside a ResourceException that is thrown back to the caller
         Exception e = ejb.getCausedByException();
         log.throwing("SdoRepositoryInteractionImpl", "execute(InteractionSpec, Record)", e);
         throw helper.createResourceException(e, currClass);
      } catch (Throwable t) {
         //Any exceptions thrown from the wrapper methods are caught here and
         //stuffed inside a ResourceException that is thrown back to the caller
         log.throwing("SdoRepositoryInteractionImpl", "execute(InteractionSpec, Record)", t);
         throw helper.createResourceException(t, currClass);
      }

      log.exiting("SdoRepositoryInteractionImpl",
         "execute(InteractionSpec, Record)", output);

      return output;
   }


   /**
    * Executes an interaction represented by the InteractionSpec. This form of invocation
    * takes an input Record and an output Record and returns a boolean indicating if the
    * execution of the Interaction has been successful.
    *
    * @param ispec InteractionSpec representing a target EIS data/function module
    * @param input Input CCI Record
    * @param output Output CCI Record
    *
    * @return boolean indicating if execution of the EIS function has been successful
    *
    * @throws ResourceException Exception if execute operation fails.
    */
   public boolean execute(InteractionSpec ispec, Record input, Record output) throws ResourceException {
      log.entering("SdoRepositoryInteractionImpl",
         "execute(InteractionSpec, Record, Record)", new Object[] {ispec, input, output});

      IndexedRecord result = (IndexedRecord) execute(ispec, input);
      boolean success = true; //assume success if no exception from above line

      if (result != null) {
         //basically "clone" the result record into the output record
         output.setRecordName(result.getRecordName());
         output.setRecordShortDescription(result.getRecordShortDescription());

         for (int index=0; index<result.size(); index++) {
            ((IndexedRecord) output).set(index, result.get(index));
         }
      }

      log.exiting("SdoRepositoryInteractionImpl",
         "execute(InteractionSpec, Record, Record)", new Boolean(success));

      return success;
   }


   /**
    * Gets the first ResourceWarning from the chain of warnings associated with this
    * Interaction instance.
    *
    * @return ResourceWarning at top of the warning chain.
    *
    * @throws ResourceException Failed to get ResourceWarnings associated with this Interaction.
    */
   public ResourceWarning getWarnings() throws ResourceException {
      log.entering("SdoRepositoryInteractionImpl", "getWarnings()");

      NotSupportedException nse = new NotSupportedException("getWarnings()");

      log.throwing("SdoRepositoryInteractionImpl", "getWarnings()", nse);

      throw nse;
   }


   /**
    * Clears all the warnings reported by this Interaction instance. After a call to this method,
    * the method getWarnings will return null until a new warning is reported for this Interaction.
    *
    * @throws ResourceException Failed to clear ResourceWarnings associated with this Interaction.
    */
   public void clearWarnings() throws ResourceException {
      log.entering("SdoRepositoryInteractionImpl", "clearWarnings()");

      NotSupportedException nse = new NotSupportedException("clearWarnings()");

      log.throwing("SdoRepositoryInteractionImpl", "clearWarnings()", nse);

      throw nse;
   }
}
