package com.ibm.websphere.samples.cmpa.sdorep.rar;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-H74,   5724-L01, 5655-N53, 5724-I82, 5655-R15 (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.3 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/sdorep/rar/SdoRepositoryObjectInputStream.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 4/13/06 22:11:09 [8/24/10 06:41:37]
//
//==============================================================================

import java.io.InputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;


/**
 * This subclass of ObjectInputStream handles deserialization of data from the
 * SDO repository.  Specifically, it addresses the case where the type of object
 * being deserialized isn't available to the RAR's class loader, such as would
 * be the case if the object is a class from a customer's application.  The
 * class object for the type of object being read is passed by the caller
 * (whose class loader can load the class), and used during resolveClass.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 */
public class SdoRepositoryObjectInputStream extends ObjectInputStream {
   Class beanDataType = null;

   /**
    * Constructor
    *
    * @param stream An InputStream, as returned by SDORepository.getInputStream
    * @param type   The type of object to be read.  Can be null, if the type is
    *               a standard J2SE type or otherwise a type that would be known
    *               to the RAR file's class loader (such as a type defined in
    *               the WebSphere lib directory).
    */
   public SdoRepositoryObjectInputStream(InputStream stream, Class type) throws IOException {
      super(stream);
      beanDataType = type;
   }


   /**
    * Resolves the Class that corresponds to the given descriptor.  This uses
    * the Class object passed in to the constructor, or otherwise delegates to
    * the default ObjectInputStream implementation.
    *
    * @param descriptor Description of the type of object to be deserialized
    *
    * @return The Class of the object being deserialized.
    */
   public Class resolveClass(ObjectStreamClass descriptor) throws IOException, ClassNotFoundException {
      Class resolvedClass = null;

      if ((beanDataType != null) && beanDataType.getName().equals(descriptor.getName())) {
         resolvedClass = beanDataType;
      } else {
         resolvedClass = super.resolveClass(descriptor);
      }

      return resolvedClass;
   }
}
