package com.ibm.websphere.samples.cmpa.sdorep.rar;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-H74,   5724-L01, 5655-N53, 5724-I82, 5655-R15 (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.7 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/sdorep/rar/SdoRepositoryWrapper.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 4/13/06 22:10:40 [8/24/10 06:41:35]
//
//==============================================================================

import com.ibm.wsspi.sdo.config.repository.RepositoryException;
import com.ibm.wsspi.sdo.config.repository.RepositoryResourceNotFoundException;
import com.ibm.wsspi.sdo.config.repository.SDORepository;

import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import java.util.List;

import javax.ejb.EJBException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.NoSuchEntityException;
import javax.ejb.RemoveException;


/**
 * This class encapsulates all usage of the SDO Repository.  The CCI Interaction
 * delegates to this class to interact with the repository, which serves as the
 * backend datastore (which is implemented as a transactional CMP entity bean).
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 */
public class SdoRepositoryWrapper {
   private static SDORepository sdoRepository = null;

   /**
    * Default constructor - initializes SDO Repository.
    */
   public SdoRepositoryWrapper() {
      if (sdoRepository == null) {
         sdoRepository = SDORepository.instance();
      }
   }

   //Generally useful CRUD methods

   /**
    * Insert EJB entity into the SDO repository.
    *
    * @param key The entity EJB key
    * @param obj The wrapper object containing the values of the CMP fields
    *            corresponding to the specified key
    *
    * @throws DuplicateKeyException Thrown if an object already exists with
    *                               the specified key
    */
   public void ejbCreate(String key, Object obj) throws DuplicateKeyException {
      try {
         OutputStream stream = sdoRepository.getOutputStream(key, false);
         ObjectOutputStream objStream = new ObjectOutputStream(stream);
         objStream.writeObject(obj);
         objStream.close();
      } catch (RepositoryException dup) {
         throw new DuplicateKeyException(key);
      } catch (Exception e) {
         throw new EJBException(e);
      }
   }


   /**
    * Retrieve EJB entity from the SDO repository.
    *
    * @param key  The entity EJB key
    * @param type The type of class being found
    *
    * @return     The wrapper object containing the values of the CMP fields
    *             corresponding to the specified key
    *
    * @throws ObjectNotFoundException Thrown if an object does not exist with
    *                                 the specified key
    */
   public Object ejbFindByPrimaryKey(String key, Class type) throws ObjectNotFoundException {
      Object bean = null;

      try {
         InputStream stream = sdoRepository.getInputStream(key);
         SdoRepositoryObjectInputStream objStream = new SdoRepositoryObjectInputStream(stream, type);
         bean = objStream.readObject();
         objStream.close();
      } catch (RepositoryResourceNotFoundException notFound) {
         throw new ObjectNotFoundException(key);
      } catch (Exception e) {
         throw new EJBException(e);
      }

      return bean;
   }


   /**
    * Update the EJB entity into the SDO repository.
    *
    * @param key The entity EJB key
    * @param obj The wrapper object containing the values of the CMP fields
    *            corresponding to the specified key
    *
    * @throws NoSuchEntityException Thrown if an object does not exist with
    *                               the specified key
    */
   public void ejbStore(String key, Object obj) throws NoSuchEntityException {
      if (exists(key)) try {
         OutputStream stream = sdoRepository.getOutputStream(key, true);
         ObjectOutputStream objStream = new ObjectOutputStream(stream);
         objStream.writeObject(obj);
         objStream.close();
      } catch (Exception e) {
         throw new EJBException(e);
      } else {
         throw new NoSuchEntityException(key);
      }
   }


   /**
    * Delete the EJB entity from the SDO repository.
    *
    * @param key The entity EJB key
    *
    * @throws RemoveException Thrown if an object does not exist with the
    *                         specified key
    */
   public void ejbRemove(String key) throws RemoveException {
      if (exists(key)) try {
         sdoRepository.remove(key, false);
      } catch (Exception e) {
         throw new EJBException(e);
      } else {
         throw new RemoveException(key);
      }
   }


   /**
    * Retrieves all EJB entities from the SDO repository.
    *
    * @return    A list of the keys for all entries in the repository
    */
   public List ejbFindAll() {
      return sdoRepository.listResources();
   }


   /**
    * Determines if EJB entity exists in the SDO repository.
    *
    * @param key The entity EJB key
    *
    * @return    Indicator of whether an object exists with the specified key
    */
   public boolean exists(String key) {
      return sdoRepository.getTimestamp(key) != null;
   }
}
