package com.ibm.websphere.samples.cmpa.websphere_deploy.cci;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.9 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/websphere_deploy/cci/AccountCCIUserDefinedPushDownMethodsImpl.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 11/15/07 14:47:05 [8/24/10 06:41:35]
//
//==============================================================================

import com.ibm.wbiserver.cmpa.WSProceduralPushDownHelper;
import com.ibm.websphere.samples.cmpa.AccountCCI;
import com.ibm.websphere.samples.cmpa.websphere_deploy.AccountCCIUserDefinedPushDownMethods;

import com.ibm.websphere.appprofile.accessintent.AccessIntent;

import java.util.logging.Logger;
import javax.resource.ResourceException;
import javax.resource.cci.IndexedRecord;
import javax.resource.cci.Record;

import com.ibm.websphere.samples.cmpa.AccountWrapper;
import com.ibm.websphere.samples.cmpa.InsufficientFundsException;


/**
 * CCI implementation of the UserDefinedPushDownMethods interface
 * for the AccountCCI CMP entity bean.
 * Roughly speaking, this is where you would put the connectivity code that
 * would have been in the bean implementation class of a BMP entity bean.
 *
 * @see com.ibm.websphere.ejbpersistence.EJBToRAAdapter
 */
public class AccountCCIUserDefinedPushDownMethodsImpl
   implements AccountCCIUserDefinedPushDownMethods
{
   private Logger log = null;

   private WSProceduralPushDownHelper helper = null;

   // *** insert any necessary additional fields here ***


   /**
    * Constructor for AccountCCIUserDefinedPushDownMethodsImpl.  JRAS tracing
    * is initialized here.  To turn on trace for this class, set a trace string
    * such as "com.ibm.websphere.samples.cmpa.websphere_deploy.cci.*=all=enabled"
    * in your server's Diagnostic Trace Service (either via wsadmin or the
    * admin console), and the output will go to your server's trace.log.
    *
    * @param pdHelper WSProceduralPushDownHelper instance,
    *        used by the push down method implementations
    */
   public AccountCCIUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper pdHelper) {
      helper = pdHelper;

      log = Logger.getLogger("com.ibm.websphere.samples.cmpa.websphere_deploy.cci");
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "AccountCCIUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)", pdHelper);

      // *** insert any necessary additional constructor code here ***

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "AccountCCIUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)");
   }


   /**
    * User-defined push-down method <code>ejbFindByPrimaryKey</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByPrimaryKey(
      String arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)",
         new Object[] {arg0, accessIntent, connection});

      Record returnValue = null;
      javax.ejb.ObjectNotFoundException notFound = null;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbFindByPrimaryKey");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         input.add(0, arg0);
         input.add(1, AccountWrapper.class);

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         AccountWrapper wrapper = (AccountWrapper) output.get(0);

         // Add records to returnValue record as described in this method's JavaDoc

         IndexedRecord beanData = helper.createCCIIndexedRecord();
         beanData.add(0, arg0);
         beanData.add(1, wrapper.getBalance());

         returnValue = helper.createCCIRecord(beanData);

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            notFound = (javax.ejb.ObjectNotFoundException) cause;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (notFound != null) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", notFound);

         throw notFound;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * Helper method to get the bean data for a given primary key, used by other
    * push-down methods.
    */
   private Object getBeanData(String key, AccessIntent accessIntent, Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      IndexedRecord record = (IndexedRecord) ejbFindByPrimaryKey(key, accessIntent, connection);
      return record.get(0);
   }


   /**
    * Helper method to get the balance for a given primary key, used by other
    * push-down methods.
    */
   private Float getBalance(String key, AccessIntent accessIntent, Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      IndexedRecord beanData = (IndexedRecord) getBeanData(key, accessIntent, connection);
      Float balance = (Float) beanData.get(1);
      return balance;
   }


   /**
    * User-defined push-down method <code>ejbFindByBalanceThreshold</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByBalanceThreshold(
      float arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)",
         new Object[] {new Float(arg0), accessIntent, connection});

      //EJB-QL: select object(o) from AccountCCI o where o.balance >= ?1

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbFindAll");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record


         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         // Add records to returnValue record as described in this method's JavaDoc

         //this is inefficient, but it works - we're calling separate finders
         //against each key we got back from the above CCI interaction, since
         //this resource adapter doesn't offer a way to get multiple objects
         //in one fetch (it only can get their keys in a single fetch).
         for (int index=0; index < output.size(); index++) {
            String key = (String) output.get(index);
            IndexedRecord beanData = (IndexedRecord) getBeanData(key, accessIntent, connection);
            Float balance = (Float) beanData.get(1);
            if (balance.floatValue() >= arg0) returnValue.add(beanData);
         }

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (javax.ejb.FinderException fe) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", fe);

         throw fe;
      } catch (ResourceException re) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbCreate</code>.
    * If the bean already exists in the backend datastore, then you should
    * throw a javax.ejb.DuplicateKeyException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.CreateException Thrown if an exception occurs creating bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbCreate(
      AccountCCI bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.CreateException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountCCI, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      javax.ejb.DuplicateKeyException dupKey = null;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbCreate");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         input.add(0, bean.getId());
         input.add(1, new AccountWrapper(bean.getId(), new Float(bean.getBalance())));

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record



         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.DuplicateKeyException) {
            dupKey = (javax.ejb.DuplicateKeyException) cause;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "ejbCreate(AccountCCI, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountCCI, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (dupKey != null) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountCCI, AccessIntent, Object)", dupKey);

         throw dupKey;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountCCI, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>credit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public float credit(
      float arg0,
      AccountCCI bean,
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "credit(float, AccountCCI, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      float returnValue = 0;
      boolean success = true;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbStore");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         String key = bean.getId();
         Float balance = getBalance(key, accessIntent, connection);
         float newBalance = balance.floatValue()+arg0;

         input.add(0, key);
         input.add(1, new AccountWrapper(key, new Float(newBalance)));

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         //Adjust the following line as appropriate
         returnValue = newBalance;

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record

         //Since credit is marked preflush=true, that means an ejbStore did
         //not occur before getting here, so we need to explicitly set the
         //balance CMP field to the new value, so that it will be correct
         //when the ejbStore happens at tran commit time.
         bean.setBalance(returnValue);

      } catch (javax.ejb.ObjectNotFoundException notFound) {
         success = false;
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.NoSuchEntityException) {
            success = false;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "credit(float, AccountCCI, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "credit(float, AccountCCI, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "credit(float, AccountCCI, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "credit(float, AccountCCI, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbSelectAllBalances</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that ejbSelect methods can be defined to return either individual
    * CMP fields or entire beans.  For the former type, the inner record
    * should only contain the single CMP field.  For the latter type, the
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbSelectAllBalances(
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //EJB-QL: select o.balance from AccountCCI o

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbFindAll");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record


         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         // Add records to returnValue record as described in this method's JavaDoc

         //this is inefficient, but it works - we're calling separate finders
         //against each key we got back from the above CCI interaction, since
         //this resource adapter doesn't offer a way to get multiple objects
         //in one fetch (it only can get their keys in a single fetch).
         for (int index=0; index < output.size(); index++) {
            String key = (String) output.get(index);
            Float balance = getBalance(key, accessIntent, connection);
            Record fieldData = helper.createCCIRecord(balance);
            returnValue.add(fieldData);
         }

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (javax.ejb.FinderException fe) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", fe);

         throw fe;
      } catch (ResourceException re) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbStore</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param bean Reference to the bean implementation class, containing the "new" field values
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param oldValues IndexedRecord holding original values of CMP fields.  If
    * the accessIntent indicates that optimistic concurrency control is being
    * used, then this holds the values of the CMP fields as they were when the
    * bean was loaded.  If the values have changed since then in the backend
    * datastore, then you should throw a javax.ejb.NoSuchEntityException.
    * The order of the CMP fields in the IndexedRecord is as follows:
    *    Position #0: id
    *    Position #1: balance
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.NoSuchEntityException Thrown if bean no longer exists
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbStore(
      AccountCCI bean,
      AccessIntent accessIntent,
      IndexedRecord oldValues,
      Object connection)
      throws javax.ejb.NoSuchEntityException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountCCI, AccessIntent, IndexedRecord, Object)",
         new Object[] {bean, accessIntent, oldValues, connection});


      float balance = 0;
      javax.ejb.NoSuchEntityException noSuchEntity = null;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbStore");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         input.add(0, bean.getId());
         input.add(1, new AccountWrapper(bean.getId(), new Float(bean.getBalance())));

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.NoSuchEntityException) {
            noSuchEntity = (javax.ejb.NoSuchEntityException) cause;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "ejbStore(AccountCCI, AccessIntent, IndexedRecord, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountCCI, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (noSuchEntity != null) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountCCI, AccessIntent, IndexedRecord, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountCCI, AccessIntent, IndexedRecord, Object)");

   }


   /**
    * User-defined push-down method <code>debit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    * @exception com.ibm.websphere.samples.cmpa.InsufficientFundsException Thrown if an exception occurs in the data logic method
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public float debit(
      float arg0,
      AccountCCI bean,
      AccessIntent accessIntent,
      Object connection)
      throws com.ibm.websphere.samples.cmpa.InsufficientFundsException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "debit(float, AccountCCI, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      float returnValue = 0;
      boolean success = true;
      InsufficientFundsException overdrawn = null;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbStore");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         String key = bean.getId();
         Float balance = getBalance(key, accessIntent, connection);
         float newBalance = balance.floatValue()-arg0;
         if (newBalance < 0) {
            throw new InsufficientFundsException(balance.toString());
         }

         input.add(0, key);
         input.add(1, new AccountWrapper(key, new Float(newBalance)));

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         //Adjust the following line as appropriate
         returnValue = newBalance;

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (InsufficientFundsException ife) {
         overdrawn = ife;
      } catch (javax.ejb.ObjectNotFoundException notFound) {
         success = false;
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.NoSuchEntityException) {
            success = false;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "debit(float, AccountCCI, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCCI, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCCI, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      if (overdrawn != null) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCCI, AccessIntent, Object)", overdrawn);

         throw overdrawn;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "debit(float, AccountCCI, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbRemove</code>.
    * If the bean does not exist in the backend datastore, then you should
    * throw a javax.ejb.RemoveException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.RemoveException Thrown if an exception occurs deleting bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbRemove(
      AccountCCI bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.RemoveException, ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountCCI, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      javax.ejb.RemoveException remEx = null;
      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbRemove");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         input.add(0, bean.getId());

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.RemoveException) {
            remEx = (javax.ejb.RemoveException) cause;
         } else {
            log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
               "ejbRemove(AccountCCI, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountCCI, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (remEx != null) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountCCI, AccessIntent, Object)", remEx);

         throw remEx;
      }

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountCCI, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>ejbHomeReset</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbHomeReset(
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)",
         new Object[] {accessIntent, connection});


      try {
         com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec iSpec = new com.ibm.websphere.samples.cmpa.sdorep.SdoRepositoryInteractionSpec();
         iSpec.setFunctionName("ejbHomeReset");
         // Specify any additional iSpec settings here

         IndexedRecord input = helper.createCCIIndexedRecord();
         // Change the above line if the resource adapter requires a different
         // form of Record, such as a MappedRecord or a Streamable Record.
         // For example, CICS ECI requires helper.createCCIStreamableRecord()

         // Begin customer-written code to populate input record

         // End customer-written code to populate input record

         IndexedRecord output = (IndexedRecord) helper.executeCCIInteraction(
            connection, iSpec, input);

         helper.returnCCIIndexedRecord(input);
         // Change the above line if using a different type of Record

         helper.returnCCIIndexedRecord(output);
         // Change the above line if using a different type of Record, or
         // remove the above line if directly returning the Record
      } catch (ResourceException re) {
         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCCIUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCCIUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)");

   }

}
