package com.ibm.websphere.samples.cmpa.websphere_deploy.custom;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.5 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/websphere_deploy/custom/AccountCustomUserDefinedPushDownMethodsImpl.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 11/15/07 14:49:39 [8/24/10 06:41:35]
//
//==============================================================================

import com.ibm.wbiserver.cmpa.WSProceduralPushDownHelper;
import com.ibm.websphere.samples.cmpa.AccountCustom;
import com.ibm.websphere.samples.cmpa.websphere_deploy.AccountCustomUserDefinedPushDownMethods;

import com.ibm.websphere.appprofile.accessintent.AccessIntent;

import java.util.logging.Logger;
import javax.resource.ResourceException;
import javax.resource.cci.IndexedRecord;
import javax.resource.cci.Record;

import java.util.HashMap;
import com.ibm.websphere.samples.cmpa.InsufficientFundsException;


/**
 * Custom implementation of the UserDefinedPushDownMethods interface
 * for the AccountCustom CMP entity bean.
 * Roughly speaking, this is where you would put the connectivity code that
 * would have been in the bean implementation class of a BMP entity bean.
 *
 * @see com.ibm.websphere.ejbpersistence.EJBToRAAdapter
 */
public class AccountCustomUserDefinedPushDownMethodsImpl
   implements AccountCustomUserDefinedPushDownMethods
{
   private Logger log = null;

   private WSProceduralPushDownHelper helper = null;

   private static HashMap map = new HashMap();


   /**
    * Constructor for AccountCustomUserDefinedPushDownMethodsImpl.  JRAS tracing
    * is initialized here.  To turn on trace for this class, set a trace string
    * such as "com.ibm.websphere.samples.cmpa.websphere_deploy.custom.*=all=enabled"
    * in your server's Diagnostic Trace Service (either via wsadmin or the
    * admin console), and the output will go to your server's trace.log.
    *
    * @param pdHelper WSProceduralPushDownHelper instance,
    *        used by the push down method implementations
    */
   public AccountCustomUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper pdHelper) {
      helper = pdHelper;

      log = Logger.getLogger("com.ibm.websphere.samples.cmpa.websphere_deploy.custom");
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "AccountCustomUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)", pdHelper);

      // *** insert any necessary additional constructor code here ***

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "AccountCustomUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)");
   }


   /**
    * User-defined push-down method <code>ejbFindByPrimaryKey</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByPrimaryKey(
      String arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)",
         new Object[] {arg0, accessIntent, connection});

      javax.ejb.ObjectNotFoundException notFound = null;
      Record returnValue = null;
      try {
         // This push-down method specified the following Backend method: retrieveAccount

         // Begin customer-written code

         IndexedRecord beanData = null;
         if (map.containsKey(arg0)) {
            beanData = helper.createCCIIndexedRecord();
            Float balance = (Float) map.get(arg0);
            beanData.add(0, arg0);
            beanData.add(1, balance);
         } else {
            notFound = new javax.ejb.ObjectNotFoundException(arg0);
         }

         // End customer-written code

         // Add records to returnValue record as described in this method's JavaDoc
         if (beanData != null) returnValue = helper.createCCIRecord(beanData);

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (notFound != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", notFound);

         throw notFound;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbFindByBalanceThreshold</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByBalanceThreshold(
      float arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)",
         new Object[] {new Float(arg0), accessIntent, connection});

      //EJB-QL: select object(o) from AccountCustom o where o.balance >= ?1

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         // This push-down method specified the following Backend method: retrieveMultipleAccounts

         // Begin customer-written code

         java.util.Set entries = map.entrySet();
         java.util.Iterator iter = entries.iterator();
         int index = 0;
         while (iter.hasNext()) {
            java.util.Map.Entry entry = (java.util.Map.Entry) iter.next();
            Float balance = (Float) entry.getValue();
            if (balance.floatValue() >= arg0) {
               String id = (String) entry.getKey();
               IndexedRecord beanData = helper.createCCIIndexedRecord();
               beanData.add(0, id);
               beanData.add(1, balance);
               returnValue.add(index++, beanData);
            }
         }

         // End customer-written code

         // Add records to returnValue record as described in this method's JavaDoc

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbCreate</code>.
    * If the bean already exists in the backend datastore, then you should
    * throw a javax.ejb.DuplicateKeyException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.CreateException Thrown if an exception occurs creating bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbCreate(
      AccountCustom bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.CreateException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountCustom, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      javax.ejb.DuplicateKeyException dupKey = null;
      try {
         // This push-down method specified the following Backend method: createAccount

         // Begin customer-written code

         String id = bean.getId();

         //might want this in a synchronized block...
         float balance = bean.getBalance();
         if (map.containsKey(id)) {
            dupKey = new javax.ejb.DuplicateKeyException(id);
         } else {
            map.put(id, new Float(balance));
         }

         // End customer-written code

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountCustom, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (dupKey != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountCustom, AccessIntent, Object)", dupKey);

         throw dupKey;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountCustom, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>credit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public float credit(
      float arg0,
      AccountCustom bean,
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "credit(float, AccountCustom, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      float returnValue = 0;
      javax.ejb.NoSuchEntityException noSuchEntity = null;
      InsufficientFundsException overdrawn = null;
      try {
         // This push-down method specified the following Backend method: creditAccount

         // Begin customer-written code

         String id = bean.getId();
         float balance = bean.getBalance();

         //might want this in a synchronized block...
         if (map.containsKey(id)) {
            returnValue = balance + arg0;
            map.put(id, new Float(returnValue));

            //Since credit is marked preflush=true, that means an ejbStore did
            //not occur before getting here, so we need to explicitly set the
            //balance CMP field to the new value, so that it will be correct
            //when the ejbStore happens at tran commit time.
            bean.setBalance(returnValue);
         } else {
            noSuchEntity = new javax.ejb.NoSuchEntityException(id);
         }

         // End customer-written code

         // The returnValue variable should be set to the appropriate value

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "credit(float, AccountCustom, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (noSuchEntity != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "credit(float, AccountCustom, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "credit(float, AccountCustom, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbSelectAllBalances</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that ejbSelect methods can be defined to return either individual
    * CMP fields or entire beans.  For the former type, the inner record
    * should only contain the single CMP field.  For the latter type, the
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbSelectAllBalances(
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //EJB-QL: select o.balance from AccountCustom o

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         // This push-down method specified the following Backend method: selectAccounts

         // Begin customer-written code

         java.util.Collection values = map.values();
         java.util.Iterator iter = values.iterator();
         int index = 0;
         while (iter.hasNext()) {
            Float balance = (Float) iter.next();
            Record beanData = helper.createCCIRecord(balance);
            returnValue.add(index++, beanData);
         }

         // End customer-written code

         // Add records to returnValue record as described in this method's JavaDoc

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbStore</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param bean Reference to the bean implementation class, containing the "new" field values
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param oldValues IndexedRecord holding original values of CMP fields.  If
    * the accessIntent indicates that optimistic concurrency control is being
    * used, then this holds the values of the CMP fields as they were when the
    * bean was loaded.  If the values have changed since then in the backend
    * datastore, then you should throw a javax.ejb.NoSuchEntityException.
    * The order of the CMP fields in the IndexedRecord is as follows:
    *    Position #0: id
    *    Position #1: balance
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.NoSuchEntityException Thrown if bean no longer exists
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbStore(
      AccountCustom bean,
      AccessIntent accessIntent,
      IndexedRecord oldValues,
      Object connection)
      throws javax.ejb.NoSuchEntityException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountCustom, AccessIntent, IndexedRecord, Object)",
         new Object[] {bean, accessIntent, oldValues, connection});

      float balance = 0;
      javax.ejb.NoSuchEntityException noSuchEntity = null;
      try {
         // This push-down method specified the following Backend method: updateAccount

         // Begin customer-written code

         String id = bean.getId();
         balance = bean.getBalance();

         //might want this in a synchronized block...
         if (map.containsKey(id)) {
            map.put(id, new Float(balance));
         } else {
            noSuchEntity = new javax.ejb.NoSuchEntityException(id);
         }

         // End customer-written code

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountCustom, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (noSuchEntity != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountCustom, AccessIntent, IndexedRecord, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountCustom, AccessIntent, IndexedRecord, Object)");

   }


   /**
    * User-defined push-down method <code>debit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    * @exception com.ibm.websphere.samples.cmpa.InsufficientFundsException Thrown if an exception occurs in the data logic method
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public float debit(
      float arg0,
      AccountCustom bean,
      AccessIntent accessIntent,
      Object connection)
      throws com.ibm.websphere.samples.cmpa.InsufficientFundsException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "debit(float, AccountCustom, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      float returnValue = 0;
      javax.ejb.NoSuchEntityException noSuchEntity = null;
      InsufficientFundsException overdrawn = null;
      try {
         // This push-down method specified the following Backend method: debitAccount

         // Begin customer-written code

         String id = bean.getId();
         float balance = bean.getBalance();

         //might want this in a synchronized block...
         if (map.containsKey(id)) {
            if (balance >= arg0) {
               returnValue = balance - arg0;
               map.put(id, new Float(returnValue));
            } else {
               overdrawn = new InsufficientFundsException(""+balance);
            }
         } else {
            noSuchEntity = new javax.ejb.NoSuchEntityException(id);
         }

         // End customer-written code

         // The returnValue variable should be set to the appropriate value

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCustom, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (noSuchEntity != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCustom, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      if (overdrawn != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "debit(float, AccountCustom, AccessIntent, Object)", overdrawn);

         throw overdrawn;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "debit(float, AccountCustom, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbRemove</code>.
    * If the bean does not exist in the backend datastore, then you should
    * throw a javax.ejb.RemoveException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.RemoveException Thrown if an exception occurs deleting bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbRemove(
      AccountCustom bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.RemoveException, ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountCustom, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      javax.ejb.RemoveException notThere = null;
      try {
         // This push-down method specified the following Backend method: deleteAccount

         // Begin customer-written code

         String id = bean.getId();

         //might want this in a synchronized block...
         if (id.equals("*")) {
            map.clear(); //lets us test remove where item no longer in backend
         } else if (map.containsKey(id)) {
            if (id.equals("fail")) {
               notThere = new javax.ejb.RemoveException(id);
            } else {
               map.remove(id);
            }
         } else {
            notThere = new javax.ejb.RemoveException(id);
         }

         // End customer-written code

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountCustom, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (notThere != null) {
         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountCustom, AccessIntent, Object)", notThere);

         throw notThere;
      }

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountCustom, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>ejbHomeReset</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbHomeReset(
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)",
         new Object[] {accessIntent, connection});


      try {
         // This push-down method specified the following Backend method: resetAccounts

         // Begin customer-written code

         //might want this in a synchronized block...
         map.clear();

         // End customer-written code

      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountCustomUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountCustomUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)");

   }

}
