package com.ibm.websphere.samples.cmpa.websphere_deploy.jaxrpc;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.10 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/websphere_deploy/jaxrpc/AccountJAXRPCUserDefinedPushDownMethodsImpl.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 11/15/07 14:54:03 [8/24/10 06:41:36]
//
//==============================================================================

import com.ibm.wbiserver.cmpa.WSProceduralPushDownHelper;
import com.ibm.websphere.samples.cmpa.AccountJAXRPC;
import com.ibm.websphere.samples.cmpa.websphere_deploy.AccountJAXRPCUserDefinedPushDownMethods;

import com.ibm.websphere.appprofile.accessintent.AccessIntent;

import java.util.logging.Logger;
import javax.resource.ResourceException;
import javax.resource.cci.IndexedRecord;
import javax.resource.cci.Record;

import com.ibm.websphere.samples.cmpa.AccountWrapper;
import com.ibm.websphere.samples.cmpa.InsufficientFundsException;


/**
 * JAX-RPC implementation of the UserDefinedPushDownMethods interface
 * for the AccountJAXRPC CMP entity bean.
 * Roughly speaking, this is where you would put the connectivity code that
 * would have been in the bean implementation class of a BMP entity bean.
 *
 * @see com.ibm.websphere.ejbpersistence.EJBToRAAdapter
 */
public class AccountJAXRPCUserDefinedPushDownMethodsImpl
   implements AccountJAXRPCUserDefinedPushDownMethods
{
   private Logger log = null;

   private WSProceduralPushDownHelper helper = null;

   javax.xml.rpc.Service service = null;
   javax.xml.namespace.QName port = null;
   String endpointAddress = null;

   private boolean useServiceRef = false;


   /**
    * Constructor for AccountJAXRPCUserDefinedPushDownMethodsImpl.  JRAS tracing
    * is initialized here.  To turn on trace for this class, set a trace string
    * such as "com.ibm.websphere.samples.cmpa.websphere_deploy.jaxrpc.*=all=enabled"
    * in your server's Diagnostic Trace Service (either via wsadmin or the
    * admin console), and the output will go to your server's trace.log.
    *
    * @param pdHelper WSProceduralPushDownHelper instance,
    *        used by the push down method implementations
    */
   public AccountJAXRPCUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper pdHelper) {
      helper = pdHelper;

      log = Logger.getLogger("com.ibm.websphere.samples.cmpa.websphere_deploy.jaxrpc");
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "AccountJAXRPCUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)", pdHelper);

      // Initialize service, port, and endpointAddress here based on META-INF/wsdl/AccountBackend.wsdl
      // Generally you would initialize the service via a java:comp lookup.
      // The port is a QName corresponding to the wsdl:portType in the WSDL.
      // The endpointAddress is a string for the soap:address in the WSDL.

      // Call helper.registerJAXRPCTypeMapping for any user-defined types used
      // in any operations called via invokeJAXRPCCall.  This includes custom
      // types passed as parameters, expected as return values, or declared as
      // exceptions that the web service operation might raise.

      // Alternatively, you could lazy-initialize such values in a method called
      // from each push-down method (such as "if (service == null) init();")

      // *** insert any necessary additional constructor code here ***

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "AccountJAXRPCUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)");
   }


   private void init() throws ResourceException {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl", "init()");

      try {
//       If we use the service-ref, registerJAXRPCTypeMapping throws an UnsupportedOperationException
         if (useServiceRef) {
            javax.naming.InitialContext ctx = new javax.naming.InitialContext();
            service = (javax.xml.rpc.Service) ctx.lookup("java:comp/env/service/AccountBackendService");
         } else {
            javax.xml.namespace.QName serviceName = new javax.xml.namespace.QName("AccountBackendService");
            service = javax.xml.rpc.ServiceFactory.newInstance().createService(serviceName);
         }



         port = new javax.xml.namespace.QName("AccountBackend");
         endpointAddress = "http://localhost:9080/cmpaSampleEJBRouter/services/AccountBackend";

         //register custom types (AccountWrapper and InsufficientFundsException)
         javax.xml.namespace.QName accountWrapper = new javax.xml.namespace.QName("http://cmpa.samples.websphere.ibm.com", "AccountWrapper");
         helper.registerJAXRPCTypeMapping(service, accountWrapper, AccountWrapper.class);

         javax.xml.namespace.QName insufficientFundsException = new javax.xml.namespace.QName("http://cmpa.samples.websphere.ibm.com", "InsufficientFundsException");
         helper.registerJAXRPCTypeMapping(service, insufficientFundsException, InsufficientFundsException.class);
      } catch (Exception e) {
         e.printStackTrace();

         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "init()", re);

         throw re;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl", "init()");
   }


   /**
    * User-defined push-down method <code>ejbFindByPrimaryKey</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByPrimaryKey(
      String arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)",
         new Object[] {arg0, accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      Record returnValue = null;
      javax.ejb.ObjectNotFoundException notFound = null;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("retrieveAccount");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://cmpa.samples.websphere.ibm.com", "AccountWrapper");

         int paramCount = 1; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = arg0;

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);
         AccountWrapper wrapper = (AccountWrapper) result;

         // Add records to returnValue record as described in this method's JavaDoc

         IndexedRecord beanData = helper.createCCIIndexedRecord();
         beanData.add(0, arg0);
         beanData.add(1, wrapper.getBalance());

         returnValue = helper.createCCIRecord(beanData);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            notFound = (javax.ejb.ObjectNotFoundException) cause;
         } else {
            log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
               "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (notFound != null) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", notFound);

         throw notFound;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbFindByBalanceThreshold</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByBalanceThreshold(
      float arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)",
         new Object[] {new Float(arg0), accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      //EJB-QL: select object(o) from AccountJAXRPC o where o.balance >= ?1

      if (service == null) init();

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("retrieveMultipleAccounts");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://xml.apache.org/xml-soap", "Vector");

         int paramCount = 1; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = new Float(arg0);

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

         // Add records to returnValue record as described in this method's JavaDoc

         java.util.Collection accounts = (java.util.Collection) result;
         java.util.Iterator iter = accounts.iterator();
         while (iter.hasNext()) {
            AccountWrapper wrapper = (AccountWrapper) iter.next();
            IndexedRecord beanData = helper.createCCIIndexedRecord();
            beanData.add(0, wrapper.getId());
            beanData.add(1, wrapper.getBalance());
            returnValue.add(beanData);
         }

      } catch (ResourceException re) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbCreate</code>.
    * If the bean already exists in the backend datastore, then you should
    * throw a javax.ejb.DuplicateKeyException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.CreateException Thrown if an exception occurs creating bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbCreate(
      AccountJAXRPC bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.CreateException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountJAXRPC, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      javax.ejb.DuplicateKeyException dupKey = null;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("createAccount");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = null;

         int paramCount = 2; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = bean.getId();
         params[1] = new Float(bean.getBalance());

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.DuplicateKeyException) {
            dupKey = (javax.ejb.DuplicateKeyException) cause;
         } else {
            log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
               "ejbCreate(AccountJAXRPC, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountJAXRPC, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (dupKey != null) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountJAXRPC, AccessIntent, Object)", dupKey);

         throw dupKey;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountJAXRPC, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>credit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public float credit(
      float arg0,
      AccountJAXRPC bean,
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "credit(float, AccountJAXRPC, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      float returnValue = 0;
      boolean success = true;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("creditAccount");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://www.w3.org/2001/XMLSchema", "float");

         int paramCount = 2; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = bean.getId();
         params[1] = new Float(arg0);

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

         //Adjust the following line as appropriate
         returnValue = ((Float)result).floatValue();

         //Since credit is marked preflush=true, that means an ejbStore did
         //not occur before getting here, so we need to explicitly set the
         //balance CMP field to the new value, so that it will be correct
         //when the ejbStore happens at tran commit time.
         bean.setBalance(returnValue);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            success = false;
         } else {
            log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
               "credit(float, AccountJAXRPC, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "credit(float, AccountJAXRPC, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "credit(float, AccountJAXRPC, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "credit(float, AccountJAXRPC, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbSelectAllBalances</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that ejbSelect methods can be defined to return either individual
    * CMP fields or entire beans.  For the former type, the inner record
    * should only contain the single CMP field.  For the latter type, the
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbSelectAllBalances(
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      //EJB-QL: select o.balance from AccountJAXRPC o

      if (service == null) init();

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("selectAccounts");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://xml.apache.org/xml-soap", "Vector");

         int paramCount = 0; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         // *** Place code here like: params[X] = argX; ***

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

         // Add records to returnValue record as described in this method's JavaDoc

         java.util.Collection balances = (java.util.Collection) result;
         java.util.Iterator iter = balances.iterator();
         int count = 0;
         while (iter.hasNext()) {
            Float balance = (Float) iter.next();
            Record balanceRecord = helper.createCCIRecord(balance);
            returnValue.add(count++, balanceRecord);
         }

      } catch (ResourceException re) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbStore</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param bean Reference to the bean implementation class, containing the "new" field values
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param oldValues IndexedRecord holding original values of CMP fields.  If
    * the accessIntent indicates that optimistic concurrency control is being
    * used, then this holds the values of the CMP fields as they were when the
    * bean was loaded.  If the values have changed since then in the backend
    * datastore, then you should throw a javax.ejb.NoSuchEntityException.
    * The order of the CMP fields in the IndexedRecord is as follows:
    *    Position #0: id
    *    Position #1: balance
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.NoSuchEntityException Thrown if bean no longer exists
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbStore(
      AccountJAXRPC bean,
      AccessIntent accessIntent,
      IndexedRecord oldValues,
      Object connection)
      throws javax.ejb.NoSuchEntityException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountJAXRPC, AccessIntent, IndexedRecord, Object)",
         new Object[] {bean, accessIntent, oldValues, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      float balance = 0;
      boolean failure = false;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("updateAccount");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://www.w3.org/2001/XMLSchema", "boolean");

         int paramCount = 2; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = bean.getId();
         balance = bean.getBalance();
         params[1] = new Float(balance);

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

         failure = ((Boolean) result).booleanValue();

      } catch (ResourceException re) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountJAXRPC, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountJAXRPC, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (failure) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountJAXRPC, AccessIntent, IndexedRecord, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountJAXRPC, AccessIntent, IndexedRecord, Object)");

   }


   /**
    * User-defined push-down method <code>debit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    * @exception com.ibm.websphere.samples.cmpa.InsufficientFundsException Thrown if an exception occurs in the data logic method
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public float debit(
      float arg0,
      AccountJAXRPC bean,
      AccessIntent accessIntent,
      Object connection)
      throws com.ibm.websphere.samples.cmpa.InsufficientFundsException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "debit(float, AccountJAXRPC, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      float returnValue = 0;
      boolean success = true;
      InsufficientFundsException overdrawn = null;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("debitAccount");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://www.w3.org/2001/XMLSchema", "float");

         int paramCount = 2; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = bean.getId();
         params[1] = new Float(arg0);

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);
         //Adjust the following line as appropriate
         returnValue = ((Float)result).floatValue();
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            success = false;
         } else if (cause instanceof InsufficientFundsException) {
            overdrawn = (InsufficientFundsException) cause;
         } else {
            log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
               "debit(float, AccountJAXRPC, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "debit(float, AccountJAXRPC, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "debit(float, AccountJAXRPC, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      if (overdrawn != null) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "debit(float, AccountJAXRPC, AccessIntent, Object)", overdrawn);

         throw overdrawn;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "debit(float, AccountJAXRPC, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbRemove</code>.
    * If the bean does not exist in the backend datastore, then you should
    * throw a javax.ejb.RemoveException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.RemoveException Thrown if an exception occurs deleting bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbRemove(
      AccountJAXRPC bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.RemoveException, ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountJAXRPC, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      boolean failure = true;
      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("deleteAccount");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = new javax.xml.namespace.QName("http://www.w3.org/2001/XMLSchema", "boolean");

         int paramCount = 1; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         params[0] = bean.getId();

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

         failure = ((Boolean) result).booleanValue();

      } catch (ResourceException re) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountJAXRPC, AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountJAXRPC, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (failure) {
         javax.ejb.RemoveException removeFailure = new javax.ejb.RemoveException(bean.getId());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountJAXRPC, AccessIntent, Object)", removeFailure);

         throw removeFailure;
      }

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountJAXRPC, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>ejbHomeReset</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbHomeReset(
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //The following code uses the JAX-RPC Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) JAX-RPC
      //stubs (produced by a tool such as wscompile or WSDL2Java) if desired.

      if (service == null) init();

      try {

         javax.xml.namespace.QName operation = new javax.xml.namespace.QName("resetAccounts");

         // Change line below if operation returns a response (that is, if it is non-void)
         // For example, if it returns a float, then set returnType as follows:
         // returnType = new QName("http://www.w3.org/2001/XMLSchema", "float");
         javax.xml.namespace.QName returnType = null;

         int paramCount = 0; //change to actual param count

         Object[] params = new Object[paramCount];


         // Begin customer-written code to populate params array

         // *** Place code here like: params[X] = argX; ***

         // End customer-written code to populate params array


         Object result = helper.invokeJAXRPCCall(
            service, port, operation, returnType, endpointAddress, params);

      } catch (ResourceException re) {
         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountJAXRPCUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountJAXRPCUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)");

   }

}
