package com.ibm.websphere.samples.cmpa.websphere_deploy.sca;

//==============================================================================
//
// "This program may be used, executed, copied, modified and distributed without
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724-L01, 5655-N53, 5724-I82, 5655-R15,  (C) COPYRIGHT International Business Machines Corp., 2004, 2006
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// @(#) 1.11 WBISRVR/ws/code/samp.cmpa/src/com/ibm/websphere/samples/cmpa/websphere_deploy/sca/AccountSCAUserDefinedPushDownMethodsImpl.java, WAS.samp.cmpa, WBI70.WBISRVR, of1033.11 11/15/07 14:56:10 [8/24/10 06:41:37]
//
//==============================================================================

import com.ibm.wbiserver.cmpa.WSProceduralPushDownHelper;
import com.ibm.websphere.samples.cmpa.AccountSCA;
import com.ibm.websphere.samples.cmpa.websphere_deploy.AccountSCAUserDefinedPushDownMethods;

import com.ibm.websphere.appprofile.accessintent.AccessIntent;

import java.util.logging.Logger;
import javax.resource.ResourceException;
import javax.resource.cci.IndexedRecord;
import javax.resource.cci.Record;

import com.ibm.websphere.samples.cmpa.AccountWrapper;
import com.ibm.websphere.samples.cmpa.InsufficientFundsException;


/**
 * SCA implementation of the UserDefinedPushDownMethods interface
 * for the AccountSCA CMP entity bean.
 * Roughly speaking, this is where you would put the connectivity code that
 * would have been in the bean implementation class of a BMP entity bean.
 *
 * @see com.ibm.websphere.ejbpersistence.EJBToRAAdapter
 */
public class AccountSCAUserDefinedPushDownMethodsImpl
   implements AccountSCAUserDefinedPushDownMethods
{
   private Logger log = null;

   private WSProceduralPushDownHelper helper = null;

   // *** insert any necessary additional fields here ***


   /**
    * Constructor for AccountSCAUserDefinedPushDownMethodsImpl.  JSR 47 tracing
    * is initialized here.  To turn on trace for this class, set a trace string
    * such as "com.ibm.websphere.samples.cmpa.websphere_deploy.sca.*=all=enabled"
    * in your server's Diagnostic Trace Service (either via wsadmin or the
    * admin console), and the output will go to your server's trace.log.
    *
    * @param pdHelper WSProceduralPushDownHelper instance,
    *        used by the push down method implementations
    */
   public AccountSCAUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper pdHelper) {
      helper = pdHelper;

      log = Logger.getLogger("com.ibm.websphere.samples.cmpa.websphere_deploy.sca");
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "AccountSCAUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)", pdHelper);

      // *** insert any necessary additional constructor code here ***

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "AccountSCAUserDefinedPushDownMethodsImpl(WSProceduralPushDownHelper)");
   }


   /**
    * User-defined push-down method <code>ejbFindByPrimaryKey</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByPrimaryKey(
      String arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)",
         new Object[] {arg0, accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      Record returnValue = null;
      javax.ejb.ObjectNotFoundException notFound = null;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "retrieveAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, arg0);

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "retrieveAccount", sdo);
         AccountWrapper wrapper = (AccountWrapper) result;

         // Add records to returnValue record as described in this method's JavaDoc

         IndexedRecord beanData = helper.createCCIIndexedRecord();
         beanData.add(0, arg0);
         beanData.add(1, wrapper.getBalance());

         returnValue = helper.createCCIRecord(beanData);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            notFound = (javax.ejb.ObjectNotFoundException) cause;
         } else {
            log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
               "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (notFound != null) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbFindByPrimaryKey(String, AccessIntent, Object)", notFound);

         throw notFound;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbFindByPrimaryKey(String, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbFindByBalanceThreshold</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param arg0 Push-down method Parameter #0
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that with single-bean finders, like ejbFindByPrimaryKey, the outer
    * record will contain exactly one inner record, whereas with multi-bean
    * finders the outer record will contain zero or more inner records.  The
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbFindByBalanceThreshold(
      float arg0,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)",
         new Object[] {new Float(arg0), accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      //EJB-QL: select object(o) from AccountSCA o where o.balance >= ?1

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "retrieveMultipleAccounts");


         // Begin customer-written code to populate SDO

         sdo.set(0, new Float(arg0));

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "retrieveMultipleAccounts", sdo);

         // Add records to returnValue record as described in this method's JavaDoc

         java.util.Collection accounts = (java.util.Collection) result;
         java.util.Iterator iter = accounts.iterator();
         while (iter.hasNext()) {
            AccountWrapper wrapper = (AccountWrapper) iter.next();
            IndexedRecord beanData = helper.createCCIIndexedRecord();
            beanData.add(0, wrapper.getId());
            beanData.add(1, wrapper.getBalance());
            returnValue.add(beanData);
         }

      } catch (ResourceException re) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbFindByBalanceThreshold(float, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbFindByBalanceThreshold(float, AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbCreate</code>.
    * If the bean already exists in the backend datastore, then you should
    * throw a javax.ejb.DuplicateKeyException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.CreateException Thrown if an exception occurs creating bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbCreate(
      AccountSCA bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.CreateException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountSCA, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      javax.ejb.DuplicateKeyException dupKey = null;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "createAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, bean.getId());
         sdo.set(1, new Float(bean.getBalance()));

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "createAccount", sdo);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.DuplicateKeyException) {
            dupKey = (javax.ejb.DuplicateKeyException) cause;
         } else {
            log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
               "ejbCreate(AccountSCA, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountSCA, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (dupKey != null) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbCreate(AccountSCA, AccessIntent, Object)", dupKey);

         throw dupKey;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbCreate(AccountSCA, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>credit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public float credit(
      float arg0,
      AccountSCA bean,
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "credit(float, AccountSCA, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      float returnValue = 0;
      boolean success = true;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "creditAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, bean.getId());
         sdo.set(1, new Float(arg0));

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "creditAccount", sdo);

         //Adjust the following line as appropriate
         returnValue = ((Float)result).floatValue();

         //Since credit is marked preflush=true, that means an ejbStore did
         //not occur before getting here, so we need to explicitly set the
         //balance CMP field to the new value, so that it will be correct
         //when the ejbStore happens at tran commit time.
         bean.setBalance(returnValue);

      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            success = false;
         } else {
            log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
               "credit(float, AccountSCA, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "credit(float, AccountSCA, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "credit(float, AccountSCA, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "credit(float, AccountSCA, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbSelectAllBalances</code>.
    * For single object finders, if the bean does not exist in the backend
    * datastore, then you should throw a javax.ejb.ObjectNotFoundException.
    * For multi-object finders, if no results are found in the backend
    * datastore, then you should return an empty Record.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this finder method is a Record of Records,
    * each inner record of which contains the values of the bean's CMP fields,
    * with primitives wrapped (such as a java.lang.Integer for an int).
    * Note that ejbSelect methods can be defined to return either individual
    * CMP fields or entire beans.  For the former type, the inner record
    * should only contain the single CMP field.  For the latter type, the
    * order of the fields of the inner record must be as follows:
    *    Position #0: id
    *    Position #1: balance
    *
    * @exception javax.ejb.FinderException Thrown if an exception occurs retrieving bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public Record ejbSelectAllBalances(
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.FinderException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      //EJB-QL: select o.balance from AccountSCA o

      IndexedRecord returnValue = helper.createCCIIndexedRecord();
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "selectAccounts");


         // Begin customer-written code to populate SDO

         // *** Place code here like: sdo.set(X, argX); ***

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "selectAccounts", sdo);

         // Add records to returnValue record as described in this method's JavaDoc

         java.util.Collection balances = (java.util.Collection) result;
         java.util.Iterator iter = balances.iterator();
         int count = 0;
         while (iter.hasNext()) {
            Float balance = (Float) iter.next();
            Record balanceRecord = helper.createCCIRecord(balance);
            returnValue.add(count++, balanceRecord);
         }

      } catch (ResourceException re) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbSelectAllBalances(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbSelectAllBalances(AccessIntent, Object)", returnValue);

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbStore</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param bean Reference to the bean implementation class, containing the "new" field values
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param oldValues IndexedRecord holding original values of CMP fields.  If
    * the accessIntent indicates that optimistic concurrency control is being
    * used, then this holds the values of the CMP fields as they were when the
    * bean was loaded.  If the values have changed since then in the backend
    * datastore, then you should throw a javax.ejb.NoSuchEntityException.
    * The order of the CMP fields in the IndexedRecord is as follows:
    *    Position #0: id
    *    Position #1: balance
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.NoSuchEntityException Thrown if bean no longer exists
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbStore(
      AccountSCA bean,
      AccessIntent accessIntent,
      IndexedRecord oldValues,
      Object connection)
      throws javax.ejb.NoSuchEntityException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountSCA, AccessIntent, IndexedRecord, Object)",
         new Object[] {bean, accessIntent, oldValues, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      float balance = 0;
      boolean failure = false;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "updateAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, bean.getId());
         balance = bean.getBalance();
         sdo.set(1, new Float(balance));

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "updateAccount", sdo);

         failure = ((Boolean) result).booleanValue();

      } catch (ResourceException re) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountSCA, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountSCA, AccessIntent, IndexedRecord, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (failure) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbStore(AccountSCA, AccessIntent, IndexedRecord, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbStore(AccountSCA, AccessIntent, IndexedRecord, Object)");

   }


   /**
    * User-defined push-down method <code>debit</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param arg0 Push-down method Parameter #0
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @return The return type of this push-down method is float
    *
    * @exception com.ibm.websphere.samples.cmpa.InsufficientFundsException Thrown if an exception occurs in the data logic method
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public float debit(
      float arg0,
      AccountSCA bean,
      AccessIntent accessIntent,
      Object connection)
      throws com.ibm.websphere.samples.cmpa.InsufficientFundsException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "debit(float, AccountSCA, AccessIntent, Object)",
         new Object[] {new Float(arg0), bean, accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      float returnValue = 0;
      boolean success = true;
      InsufficientFundsException overdrawn = null;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "debitAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, bean.getId());
         sdo.set(1, new Float(arg0));

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "debitAccount", sdo);
         //Adjust the following line as appropriate
         returnValue = ((Float)result).floatValue();
      } catch (ResourceException re) {
         Throwable cause = re.getCause();
         if (cause instanceof javax.ejb.ObjectNotFoundException) {
            success = false;
         } else if (cause instanceof InsufficientFundsException) {
            overdrawn = (InsufficientFundsException) cause;
         } else {
            log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
               "debit(float, AccountSCA, AccessIntent, Object)", re);

            throw re;
         }
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "debit(float, AccountSCA, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (!success) {
         javax.ejb.NoSuchEntityException noSuchEntity = new javax.ejb.NoSuchEntityException(bean.getId());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "debit(float, AccountSCA, AccessIntent, Object)", noSuchEntity);

         throw noSuchEntity;
      }

      if (overdrawn != null) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "debit(float, AccountSCA, AccessIntent, Object)", overdrawn);

         throw overdrawn;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "debit(float, AccountSCA, AccessIntent, Object)", new Float(returnValue));

      return returnValue;
   }


   /**
    * User-defined push-down method <code>ejbRemove</code>.
    * If the bean does not exist in the backend datastore, then you should
    * throw a javax.ejb.RemoveException.
    *
    * @param bean Reference to the bean implementation class
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    * @exception javax.ejb.RemoveException Thrown if an exception occurs deleting bean data
    * @exception ResourceException Any other exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbRemove(
      AccountSCA bean,
      AccessIntent accessIntent,
      Object connection)
      throws javax.ejb.RemoveException, ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountSCA, AccessIntent, Object)",
         new Object[] {bean, accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.

      boolean failure = true;
      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "deleteAccount");


         // Begin customer-written code to populate SDO

         sdo.set(0, bean.getId());

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "deleteAccount", sdo);

         failure = ((Boolean) result).booleanValue();

      } catch (ResourceException re) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountSCA, AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountSCA, AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      if (failure) {
         javax.ejb.RemoveException removeFailure = new javax.ejb.RemoveException(bean.getId());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbRemove(AccountSCA, AccessIntent, Object)", removeFailure);

         throw removeFailure;
      }

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbRemove(AccountSCA, AccessIntent, Object)");

   }


   /**
    * User-defined push-down method <code>ejbHomeReset</code>.
    * If the bean no longer exists in the backend datastore, then you should
    * throw a javax.ejb.NoSuchEntityException.
    *
    * @param accessIntent Reference to the access intent settings to be used for this method
    * @param connection The CCI connection to the back-end system.  This
    * connection was previously obtained (by the EJBToRAAdapter implementation)
    * via connectionFactory.getConnection(connectionSpec).  For non-relational,
    * non-CCI based beans, this will be null and the user will have to manually
    * connect to the back-end system and close the connection before returning.
    *
    *
    * @exception ResourceException Any exceptions are wrapped in a
    * ResourceException.
    */
   public void ejbHomeReset(
      AccessIntent accessIntent,
      Object connection)
      throws ResourceException
   {
      log.entering("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)",
         new Object[] {accessIntent, connection});

      //The following code uses the SCA Dynamic Invocation Interface (DII).
      //You can replace it with code that uses static (strongly-typed) SCA
      //stubs (similar to an RMI client) if desired.


      try {
         commonj.sdo.DataObject sdo = helper.createSDO(
            "AccountBackend", "resetAccounts");


         // Begin customer-written code to populate SDO

         // *** Place code here like: sdo.set(X, argX); ***

         // End customer-written code to populate SDO


         Object result = helper.invokeSCAService(
            "AccountBackend", "resetAccounts", sdo);

      } catch (ResourceException re) {
         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      } catch (Exception e) {
         ResourceException re = helper.createResourceException(e, this.getClass());

         log.throwing("AccountSCAUserDefinedPushDownMethodsImpl",
            "ejbHomeReset(AccessIntent, Object)", re);

         throw re;
      }

      //Note that if the return code from the back-end datastore interaction
      //indicated that a user-defined exception (declared in the push-down
      //method's signature) should be thrown, then throw it here.

      log.exiting("AccountSCAUserDefinedPushDownMethodsImpl",
         "ejbHomeReset(AccessIntent, Object)");

   }

}
